#ifndef _FILEPARSER_H_INCLUDED___
#define _FILEPARSER_H_INCLUDED___

#include "taskd.h"

//static char* TaskTypeExtensions[] = { "sh" , "so", "class", NULL };

#define NOTRACE

class FileParser {
 private:
	int m_nPriority;
	int m_nRetryNum;
	time_t m_nDateTime;
	char *m_strAnyString;
	TaskType m_Extension;
	bool m_bValidFileName;

	void Parse(char *strFileName);
	time_t ConvertDateTime(char *str);
	inline bool IsDigit(char *str);
	inline void Cleanup();

 public:
	FileParser(char *strFileName);
	FileParser(int nPriority, time_t nDateTime, TaskType ext = TASK_TYPE_SHELL,
			int nRetryNum = 0, char *strAnyString = NULL);
	~FileParser();

	inline int GetPriority() const {
		return m_nPriority;
	} 
	inline int GetRetryNum() const {
		return m_nRetryNum;
	} 
	inline time_t GetDateTime() const {
		return m_nDateTime;
	} 
	inline char *GetAnyString() const {
		return m_strAnyString;
	} 
	inline TaskType GetExtension() const {
		return m_Extension;
	}
	inline bool IsValidFileName() {
		return m_bValidFileName;
	}
	
	 void SetDateTime(time_t nDateTime) {
		m_nDateTime = nDateTime;
	}
	void IncrementRetryNum() {
		m_nRetryNum++;
	}

	char *CreateFileName();


	bool LessThan(const FileParser * fp) const {
#ifndef NOTRACE
		trace("LessThan (%p,%p): %d %d %d %s %d <-> %d %d %d %s %d", this, fp,
				m_nPriority, m_nDateTime, m_nRetryNum, m_strAnyString, m_Extension,
				fp->GetPriority(), fp->GetDateTime(), fp->GetRetryNum(), fp->GetAnyString(), fp->GetExtension());
#endif
		if (m_nPriority != fp->GetPriority()) {
#ifndef NOTRACE
			trace("LessThan (%p,%p) returns: %d [priority]", this, fp, m_nPriority < fp->GetPriority());
#endif
			return m_nPriority < fp->GetPriority();
		} if (m_nDateTime != fp->GetDateTime()) {
#ifndef NOTRACE
			trace("LessThan (%p,%p) returns: %d [datetime]", this, fp, m_nDateTime < fp->GetDateTime());
#endif
			return m_nDateTime < fp->GetDateTime();
		}
		if (m_nRetryNum != fp->GetRetryNum()) {
#ifndef NOTRACE
			trace("LessThan (%p,%p) returns: %d [retry]", this, fp, m_nRetryNum < fp->GetRetryNum());
#endif
			return m_nRetryNum < fp->GetRetryNum();
		}
		if (NULL == m_strAnyString || NULL == fp->GetAnyString()) {
			if (m_strAnyString != fp->GetAnyString()) {
#ifndef NOTRACE
				trace("LessThan (%p,%p) returns: %d [anystrNULL]", this, fp, m_strAnyString < fp->GetAnyString());
#endif
				return m_strAnyString < fp->GetAnyString();
			}
		}
		else {
			int cmp = strcmp(m_strAnyString, fp->GetAnyString());

			if (cmp != 0) {
#ifndef NOTRACE
				trace("LessThan (%p,%p) returns: %d [anystrCMP]", this, fp, cmp < 0);
#endif
				return cmp < 0;
			}
		}
#ifndef NOTRACE
		trace("LessThan (%p,%p) returns: %d [extension]", this, fp, m_Extension < fp->GetExtension());
#endif
		return m_Extension < fp->GetExtension();
	}

	bool EarlierThan(const FileParser * fp) const {
#ifndef NOTRACE
		trace("EarlierThan (%p,%p): %d %d %d %s %d <-> %d %d %d %s %d", this, fp,
				m_nPriority, m_nDateTime, m_nRetryNum, m_strAnyString, m_Extension,
				fp->GetPriority(), fp->GetDateTime(), fp->GetRetryNum(), fp->GetAnyString(), fp->GetExtension());
#endif
		if (m_nDateTime != fp->GetDateTime()) {
#ifndef NOTRACE
			trace("EarlierThan (%p,%p) returns: %d [datetime]", this, fp, m_nDateTime < fp->GetDateTime());
#endif
			return m_nDateTime < fp->GetDateTime();
		} if (m_nPriority != fp->GetPriority()) {
#ifndef NOTRACE
			trace("EarlierThan (%p,%p) returns: %d [priority]", this, fp, m_nPriority < fp->GetPriority());
#endif
			return m_nPriority < fp->GetPriority();
		}
		if (m_nRetryNum != fp->GetRetryNum()) {
#ifndef NOTRACE
			trace("EarlierThan (%p,%p) returns: %d [retry]", this, fp, m_nRetryNum < fp->GetRetryNum());
#endif
			return m_nRetryNum < fp->GetRetryNum();
		}
		if (NULL == m_strAnyString || NULL == fp->GetAnyString()) {
			if (m_strAnyString != fp->GetAnyString()) {
#ifndef NOTRACE
				trace("EarlierThan (%p,%p) returns: %d [anystrNULL]", this, fp, m_strAnyString < fp->GetAnyString());
#endif
				return m_strAnyString < fp->GetAnyString();
			}
		}
		else {
			int cmp = strcmp(m_strAnyString, fp->GetAnyString());

			if (cmp != 0) {
#ifndef NOTRACE
				trace("EarlierThan (%p,%p) returns: %d [anystrCMP]", this, fp, cmp < 0);
#endif
				return cmp < 0;
			}
		}
#ifndef NOTRACE
		trace("EarlierThan (%p,%p) returns: %d [extension]", this, fp, m_Extension < fp->GetExtension());
#endif
		return m_Extension < fp->GetExtension();
	}
};

#endif								  // _FILEPARSER_H_INCLUDED___
