/*
 * Component: taskd
 *
 * Description:
 *    Header file for so manager implementation
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#include <list>
#include <dlfcn.h>

using namespace std;

#include "imtask.h"
#include "mymutex.h"

typedef IMTask *(*fvinit) (const RETRY_PARAM &);
typedef void (*fvdestroy) (IMTask *);

class SoItem {
 private:
	char *name;
	void *handle;
	fvinit initTask;
	fvdestroy destroyTask;

	IMTask *task;

 public:
	SoItem(const char *soname, const RETRY_PARAM & param);
	~SoItem();

	int Run(int argc, char *argv[], int retry);
	void Print(int argc, char * argv[]);
	void Cleanup(int argc, char * argv[]);

	int CalcRetrySec(int retry);
	DrWatson *GetWatsonMessage(int argc, char *argv[], int retry, int returncode);

	char *GetName() {
		return name;
	}

	bool IsValid() {
		return (task != NULL);
	}

};


class SoManager {
 private:
	list < SoItem * >m_SoList;

	MyMutex m_mutex;

	typedef list < SoItem * >::iterator SoIterator;

	RETRY_PARAM rparam;

	void CalcRetryLimit() {
		rparam.nRetryLimit =
				(rparam.nMaxDelaySec - rparam.nFirstDelaySec) * rparam.nIncreaseDelayStep / rparam.nIncreaseDelaySec;
	}

	SoItem * getOrLoadSoItem(const char * soname);

 public:
	SoManager();

	~SoManager();

	int Run(const char *soname, int argc, char *argv[], int retry);
	void Print(const char *soname, int argc, char *argv[]);
	void Cleanup(const char *soname, int argc, char *argv[]);

	void SetDefaultMaxRetryNum(int newValue) {
		if (newValue != 0)
			rparam.nMaxRetryNum = newValue;
		CalcRetryLimit();
	}

	void SetDefaultMaxDelaySec(int newValue) {
		if (newValue != 0)
			rparam.nMaxDelaySec = newValue;
		CalcRetryLimit();
	}

	void SetDefaultFirstDelaySec(int newValue) {
		if (newValue != 0)
			rparam.nFirstDelaySec = newValue;
		CalcRetryLimit();
	}

	void SetDefaultIncreaseDelaySec(int newValue) {
		if (newValue != 0)
			rparam.nIncreaseDelaySec = newValue;
		CalcRetryLimit();
	}

	void SetDefaultIncreaseDelayStep(int newValue) {
		if (newValue != 0)
			rparam.nIncreaseDelayStep = newValue;
		CalcRetryLimit();
	}
};
